#!/usr/bin/env python
#coding=utf8
"""
$Id: DS18B20db.py,v 1.2 2014/05/13 11:09:30 alfille Exp $

Copyright (c) 2014 C-Bell (VAGor). All rights reserved.

Модуль обеспечивает работу с базами данных SQLite:
    -   проверку наличия БД Distiller.db;
    -   проверку наличия таблиц в ней;
    -   заполнение таблиц данными цифровых термометров DS18B20

Для работы программы должны быть установлены пакеты:
    python
"""

import sqlite3, DS18B20, threading, time

dbLock=threading.Lock()

db_name='Distiller.db'      #Имя базы данных
db_con=sqlite3.connect(db_name) #открыть БД
db_cur=db_con.cursor()      #создать курсор БД

#Событие завершения измерения DS18B20 и сохранения температур в таблице DS18B20cur
EndMeasure=threading.Event()
#Событие завершения записи температур в БД
ReadyData=threading.Event()

def matchDS18B20list():
    """Проверка соответствия списка термометров в БД перечню термометров
    на шине 1-wire"""
    db_cur.execute("SELECT ID FROM DS18B20cur")
    #Преобразование списка кортежей в список
    ListDB=[]
    for ts in db_cur:
        tss,=ts
        ListDB.append(tss)
    #Сравнение длин списков
    if len(DS18B20.list()) != len(ListDB):
        return False
    #Проверка идентичности списков
    for item in DS18B20.list():
        if not item in ListDB:
            return False
    return True

def filDS18B20():
    """Заполнение таблицы DS18B20cur в БД списком цифровых термометров"""
    #Очистка таблицы
    dbLock.acquire()
    db_cur.execute("DELETE FROM DS18B20cur")
    db_con.commit()
    dbLock.release()
    #Запись нового списка термометров DS18B20 в БД
    dbLock.acquire()
    for ts in DS18B20.list():
        db_cur.execute("INSERT INTO DS18B20cur (ID) values (?)" , (ts,))
        db_con.commit()
    dbLock.release()

def isTable(table_name):
    """Проверяется наличие таблицы с именем table_name в БД и возвращается
    соответствующее логическое значение"""
    return ((table_name,) in db_cur.execute("SELECT name FROM sqlite_master WHERE type='table' AND name=?;" , (table_name,)))

def delTable(table_name):
    """Удаление из БД таблицы с именем table_name"""
    dbLock.acquire()
    db_cur.execute("DROP TABLE IF EXISTS %s" % table_name)
    db_con.commit()
    dbLock.release()

def createDS18B20():
    """Создание таблицы, хранящей выборки температур DS18B20"""
    __SQL__="CREATE TABLE DS18B20 (DateTime text"
    for ts in DS18B20.list():
        __SQL__+=", '"+ts+"' text"
    __SQL__+=")"
    dbLock.acquire()
    db_cur.execute(__SQL__)
    dbLock.release()

#Разрешитель измерения температуры
__Measure__=False
#Разрешитель сохранения температур и времени в БД
__Write__=False

def StartMeasure():
    """Запуск процедуры измерения температур"""
    global __Measure__
    __Measure__=True
    __thread__=threading.Thread(target=Measure)
    __thread__.start()

def StopMeasure():
    """Останов процесса измерения температур"""
    global __Measure__
    __Measure__=False

def Measure():
    """Измерение температур и сохранения их в БД"""
    __con__=sqlite3.connect(db_name) #открыть БД
    __cur__=__con__.cursor()      #создать курсор БД
    while __Measure__:
        if EndMeasure.isSet(): EndMeasure.clear()
        #Сохранение значений температур DS18B20 в таблице DS18B20cur
        try:
            for ts in DS18B20.temperatures():
                ID,T=ts
                dbLock.acquire()
                __cur__.execute("UPDATE DS18B20cur SET T=? WHERE ID=?", (T,ID))
                __con__.commit()
                dbLock.release()
            EndMeasure.set()
        except Exception as e:
            __cur__.execute(u"INSERT INTO Errors (DateTime,Type,Description) VALUES (?,?,?)",(time.strftime("%Y.%m.%d %H:%M:%S"),str(type(e)),str(e)))
            __con__.commit()
 
def StartWrite():
    """Запуск записи температур в БД"""
    global __Write__
    __Write__=True
    __thread__=threading.Thread(target=Write)
    __thread__.start()

def StopWrite():
    """Останов записи температур в БД"""
    global __Write__
    __Write__=False

def Write():
    """Сохранение новых значений температур в БД"""
    __con__=sqlite3.connect(db_name) #открыть БД
    __cur__=__con__.cursor()      #создать курсор БД
    while __Write__:
        if ReadyData.isSet(): ReadyData.clear()
        EndMeasure.wait()   #Ожидание завершения измерения
        __insert__="INSERT INTO DS18B20 (DateTime"
        __values__=" VALUES ('"+time.strftime("%Y.%m.%d %H:%M:%S")+"'"
        #Сохранение значений температур DS18B20 в таблице DS18B20cur
        __cur__.execute("SELECT ID,T FROM DS18B20cur")
        for ts in __cur__:
            ID,T=ts
            __insert__+=", '"+ID+"'"
            __values__+=", '"+str(T)+"'"
        __insert__+=")"
        __values__+=")"
        #Сохранение значений в таблице DS18B20
        dbLock.acquire()
        __cur__.execute(__insert__+__values__)
        __con__.commit()
        ReadyData.set()
        dbLock.release()
 

def main():
    """Тестовая функция запускает и останавливает процесс заполнения
    таблиц данными цифровых термометров DS18B20"""
    StartMeasure()
    End=raw_input("Запуск измерения, Enter для вывода текущих результатов")
    EndMeasure.wait()
    db_cur.execute("SELECT * FROM DS18B20cur")
    for row in db_cur:
        ID, Name, T = row
        print (u"Термометр %s, T=%s" % (ID,T))
    print("")
    StartWrite()
    End=raw_input("Запуск записи в БД, Enter для завершения")
    StopMeasure()
    StopWrite()
    db_cur.execute("PRAGMA table_info(DS18B20)")
    columns=[]
    for column in db_cur:
        columns.append(column[1])
    print columns
    db_cur.execute("SELECT * FROM DS18B20")
    for row in db_cur:
        print row

#Проверка наличия таблицы DS18B20cur, создание в случае отсутствия и заполнение списком DS18B20
if not isTable('DS18B20cur'):
    print ('Таблица DS18B20cur в базе данных Distiller.db не обнаружена, создание таблицы')
    db_cur.execute("CREATE TABLE DS18B20cur (ID text,Name text, T real)")
    print ('Таблица DS18B20cur создана, заполнение таблицы списком DS18B20')
    filDS18B20()            #заполнить списком DS18B20
    delTable("DS18B20")     #удалить таблицу хранения показаний DS18B20
    #запустить процедуру определения места термометров
    print ('Cписок DS18B20 сохранен в БД')

#Проверка соответствия списка термометров в БД перечню термометров на шине 1-wire
elif not matchDS18B20list():
    print ("Список DS18B20 не соответствует списку DS18B20 на шине 1-wire, обновление списка")
    filDS18B20()
    print ('Cписок DS18B20 в БД обновлен')
    delTable("DS18B20")     #удалить таблицу хранения показаний DS18B20
    #Тут нужно определить местоположение термометров

#Проверка наличия таблицы DS18B20, создание её копии, создание в случае отсутствия и заполнение списком DS18B20
if isTable('DS18B20'):
        print("Таблица DS18B20 найдена в БД, удаление таблицы DS18B20old")
        delTable("DS18B20old")
        print("Таблица DS18B20old удалена, переименование таблицы DS18B20 в DS18B20old")
        db_cur.execute("ALTER TABLE DS18B20 RENAME TO DS18B20old")
        print("Таблица DS18B20 переименована в DS18B20old, создание новой таблицы DS18B20")
        createDS18B20()
        print("Таблица DS18B20 создана")
else:
        print("Таблица DS18B20 не найдена в БД, создание новой таблицы DS18B20")
        createDS18B20()
        print("Таблица DS18B20 создана")

#Создание (если не создана) таблицы для записи ошибок
db_cur.execute("CREATE TABLE IF NOT EXISTS Errors (DateTime,Type,Description)")
#Очистка ошибок предыдущей перегонки
db_cur.execute("DELETE FROM Errors")
db_con.commit()

if __name__=="__main__":
        main()


